unit Math2D;

interface

uses
	Windows, Messages, SysUtils, Classes, Math;

type
	PPointF = ^TPointF;

	TPointF = packed record
		x: Double;
		y: Double;
	end;

	TLine2D = packed record
		x1, y1, x2, y2: Double;
	end;

function PointF(const x, y: Double): TPointF;

function PointsEqualF(const p1, p2: TPointF): Boolean;

function PointToPointF(const p: TPoint): TPointF;

function PointFToPoint(const p: TPointF): TPoint;

function PointWithinLine(const px, py, x1, y1, x2, y2: Double): Boolean;

function PointOfAngle(const x, y, l, angle: Double): TPointF;

function Line2D(const x1, y1, x2, y2: Double): TLine2D; overload;

function Line2D(const p1, p2: TPointF): TLine2D; overload;

function LinesIntersect(const x1, y1, x2, y2, x3, y3, x4, y4: Double;
	var p: TPointF): Boolean;

function RotatePointF(const x, y, ox, oy, angle: Double): TPointF;

function RadiusCicle(const ox, oy, x, y: Double): Double;

function DistancePointsF(const x1, y1, x2, y2: Double): Double;

function DistancePointLine(const x, y: Double; l: TLine2D): Double;

function AnglePoints(const x1, y1, x2, y2: Double): Double;

procedure RotatePointFA(var x, y: Double; const ox, oy, angle: Double);

implementation

function PointF(const x, y: Double): TPointF;
begin
	Result.x := x;
	Result.y := y;
end;

function PointsEqualF(const p1, p2: TPointF): Boolean;
begin
	Result := (p1.x = p2.x) and (p1.y = p2.y);
end;

function PointToPointF(const p: TPoint): TPointF;
begin
	Result.x := p.x;
	Result.y := p.y;
end;

function PointFToPoint(const p: TPointF): TPoint;
begin
	Result.x := Round(p.x);
	Result.y := Round(p.y);
end;

// Point within Line bounding box

function PointWithinLine(const px, py, x1, y1, x2, y2: Double): Boolean;
begin
	if (px >= Min(x1, x2)) and (px <= Max(x1, x2)) and (py >= Min(y1, y2)) and
		(py <= Max(y1, y2)) then
		Result := True
	else
		Result := False
end;

// Finds a point at given length from O at Angle.

function PointOfAngle(const x, y, l, angle: Double): TPointF;
var
	sinTheta, cosTheta: Extended;
begin
	SinCos(DegToRad(angle), sinTheta, cosTheta);
	Result.x := x + l * cosTheta;
	Result.y := y + l * sinTheta;
end;

function Line2D(const x1, y1, x2, y2: Double): TLine2D;
begin
	Result.x1 := x1;
	Result.y1 := y1;
	Result.x2 := x2;
	Result.y2 := y2;
end;

function Line2D(const p1, p2: TPointF): TLine2D;
begin
	Result.x1 := p1.x;
	Result.y1 := p1.y;
	Result.x2 := p2.x;
	Result.y2 := p2.y;
end;

// Lines: [x1, y1, x2, y2], [x3, y3, x4, y4]

function LinesIntersect(const x1, y1, x2, y2, x3, y3, x4, y4: Double;
	var p: TPointF): Boolean;
var
	d, xc, yc: Double;
begin
	d := (x1 - x2) * (y3 - y4) - (y1 - y2) * (x3 - x4);
	if d = 0 then
		Result := False
	else
	begin
		xc := ((x3 - x4) * (x1 * y2 - y1 * x2) - (x1 - x2) *
			(x3 * y4 - y3 * x4)) / d;
		yc := ((y3 - y4) * (x1 * y2 - y1 * x2) - (y1 - y2) *
			(x3 * y4 - y3 * x4)) / d;
		p := PointF(xc, yc);
		if PointWithinLine(xc, yc, x1, y1, x2, y2) and
			PointWithinLine(xc, yc, x3, y3, x4, y4) then
			Result := True
		else
			Result := False;
	end;
end;

// Rotates a point around Origin point at Angle

function RotatePointF(const x, y, ox, oy, angle: Double): TPointF;
var
	sinTheta, cosTheta, xc, yc: Extended;
begin
	SinCos(DegToRad(angle), sinTheta, cosTheta);
	xc := cosTheta * (x - ox) - sinTheta * (y - oy) + ox;
	yc := sinTheta * (x - ox) + cosTheta * (y - oy) + oy;
	Result.x := xc;
	Result.y := yc;
end;

// Finds the radius of a circle with Origin point and Point

function RadiusCicle(const ox, oy, x, y: Double): Double;
var
	dx, dy: Double;
begin
	dx := x - ox;
	dy := y - oy;
	Result := Sqrt(dx * dx + dy * dy);
end;

// Distance between two points

function DistancePointsF(const x1, y1, x2, y2: Double): Double;
begin
	Result := Sqrt(Math.Power(x1 - x2, 2) + Math.Power(y1 - y2, 2));
end;

// Distance Between point and line

function DistancePointLine(const x, y: Double; l: TLine2D): Double;
begin
	Result := Abs((l.x2 - l.x1) * (l.y1 - y) - (l.x1 - x) * (l.y2 - l.y1)) /
		DistancePointsF(l.x1, l.y1, l.x2, l.y2);
end;

// Angle between two points

function AnglePoints(const x1, y1, x2, y2: Double): Double;
begin
	Result := RadToDeg(ArcTan2((y1 - y2), (x1 - x2)));
end;

// Rotates a point around Origin point at Angle

procedure RotatePointFA(var x, y: Double; const ox, oy, angle: Double);
var
	sinTheta, cosTheta, xc, yc: Extended;
begin
	SinCos(DegToRad(angle), sinTheta, cosTheta);
	xc := cosTheta * (x - ox) - sinTheta * (y - oy) + ox;
	yc := sinTheta * (x - ox) + cosTheta * (y - oy) + oy;
	x := xc;
	y := yc;
end;

end.
